﻿using System;
using SDX.Compiler;
using Mono.Cecil;
using Mono.Cecil.Cil;
using System.Linq;
using System.Collections.Generic;

public class DeadWellnessDown : IPatcherMod
{
    public bool Link(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        InsertGameStageDefinitionRead(gameModule, modModule);
        InsertDeadWellnessDown(gameModule, modModule);
        return true;
    }
    public void InsertDeadWellnessDown(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        TypeDefinition entityAlive = gameModule.Types.First(d => d.Name == "EntityAlive");
        MethodDefinition setAlive = entityAlive.Methods.First(d => d.Name == "SetAlive");
        Instruction insertIs = null;
        foreach (Instruction inst in setAlive.Body.Instructions)
        {
            if (inst.OpCode == OpCodes.Callvirt && inst.Operand.GetType() == typeof(MethodDefinition) && ((MethodDefinition)inst.Operand).FullName == "System.Single Stat::get_Value()")
            {
                if (inst.Next.OpCode == OpCodes.Ldc_R4)
                {
                    insertIs = inst.Next;

                }
            }
        }
        ILProcessor proc1 = setAlive.Body.GetILProcessor();

        TypeDefinition gameStageDef = gameModule.Types.First(d => d.Name == "GameStageDefinition");
        FieldDefinition deadWellnessDownField = gameStageDef.Fields.First(d => d.Name == "DeadWellnessDown");
        proc1.InsertBefore(insertIs, Instruction.Create(OpCodes.Ldsfld, deadWellnessDownField));
        proc1.Remove(insertIs);


    }

    public void InsertGameStageDefinitionRead(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        TypeDefinition classDef = gameModule.Types.First(d => d.Name == "GameStagesFromXml");
        MethodDefinition methodDef = classDef.Methods.First(d => d.Name == "Load");

        TypeDefinition gameStageDef = gameModule.Types.First(d => d.Name == "GameStageDefinition");
        FieldDefinition deadWellnessDownField = gameStageDef.Fields.First(d => d.Name == "DeadWellnessDown");
        Instruction insertPosition = null;
        foreach (Instruction inst in methodDef.Body.Instructions)
        {

            if (inst.OpCode == OpCodes.Stsfld && inst.Operand.GetType() == typeof(FieldDefinition) && ((FieldDefinition)inst.Operand).FullName == "System.Single GameStageDefinition::DaysAliveDifficultyBonus")
            {
                insertPosition = inst.Next;

            }
        }
        List<Instruction> jumpList = new List<Instruction>();
        foreach (Instruction inst in methodDef.Body.Instructions)
        {
            if (inst.OpCode == OpCodes.Brfalse_S && inst.Operand.GetType() == typeof(Instruction) && inst.Operand == insertPosition)
            {
                jumpList.Add(inst);
            }
        }

        ILProcessor proc1 = methodDef.Body.GetILProcessor();
        Instruction newJumpTo = Instruction.Create(OpCodes.Ldloc_S, proc1.Body.Variables[4]);
        proc1.InsertBefore(insertPosition, newJumpTo);//V_4 is XmlElement
        proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Ldsflda, deadWellnessDownField));//DeadWellnessDownへの参照を渡す

        TypeDefinition deadWellnessDownMethod = modModule.Types.First(d => d.Name == "DeadWellnessDownMethod");
        MethodDefinition setAddAttribute = deadWellnessDownMethod.Methods.First(d => d.Name == "SetAddAttribute");

        if (setAddAttribute == null)
        {
            Console.WriteLine("InsertGameStageDefinitionRead setAddAttribute == null");
        }
        Console.WriteLine("InsertGameStageDefinitionRead setAddAttribute find");
        MethodReference setAddAttributeRef = gameModule.Import(setAddAttribute);

        proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Call, setAddAttributeRef));

        foreach (Instruction jump in jumpList)
        {
            jump.Operand = newJumpTo;
        }
    }

    public bool Patch(ModuleDefinition module)
    {
        Console.WriteLine("==DeadWellnessDown Patcher===");
        Update(module);
        return true;
    }
    private void Update(ModuleDefinition module)
    {

        AddGameStageDefinitionField(module);

    }

    public void AddFieldDefinition(TypeDefinition classDef, string fieldName, FieldAttributes fieldAttribute, TypeReference fieldType)
    {
        FieldDefinition fd = new FieldDefinition(fieldName, fieldAttribute, fieldType);
        classDef.Fields.Add(fd);

    }
    public void AddFieldDefinition(TypeDefinition classDef, string fieldName, FieldAttributes fieldAttribute, TypeReference fieldType,object constantvalue)
    {
        FieldDefinition fd = new FieldDefinition(fieldName, fieldAttribute, fieldType);
        fd.Constant = constantvalue;
//        fd.
        classDef.Fields.Add(fd);

    }
    public void AddGameStageDefinitionField(ModuleDefinition module)
    {
        TypeDefinition multiBuffClass = module.Types.First(d => d.Name == "GameStageDefinition");
        TypeReference tr = module.Import(typeof(float));
        AddFieldDefinition(multiBuffClass, "DeadWellnessDown", FieldAttributes.Public | FieldAttributes.Static, tr);

    }
}
